/*
 * Copyright (C) 1996 Michael R. Elkins.
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include "mutt.h"
#include "rfc822.h"

#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

#define MAXUSERHDRS 25

ENVELOPE *parse_header (FILE *f, BODY **m, char **b, int blen, char *host)
{
  char line[10240], buffer[LONG_STRING], *p;
  ENVELOPE *e = mutt_new_envelope();
  int matched;

  line[0] = 0;
  while (fgets(buffer, sizeof(buffer)-1, f) != NULL) {

    /* Remove trailing whitespace from the line. */
    p = buffer + strlen(buffer) - 1;
    for (; p >= buffer; p--) {
      if (*p == '\n' || *p == ' ' || *p == '\t')
        *p = 0;
      else
        break;
    }

    if (buffer[0] == ' ' || buffer[0] == '\t') {
      strcat(line, " ");
      strcat(line, skip_whitespace(buffer));
    }
    else if (line[0]) {
      matched = 0;

      /* find the value of this header */
      if ((p = strchr(line, ':'))) p = skip_whitespace(p+1);

      switch(tolower(line[0])) {
      case 'b':
        if (strncasecmp(line+1, "cc:", 3) == 0) {
          rfc822_parse_adrlist(&e->bcc, p, host);
          matched = 1;
        }
        break;
      case 'c':
        if (strncasecmp (line+1, "c:", 2) == 0) {
          rfc822_parse_adrlist(&e->cc, p, host);
          matched = 1;
        }
        else if (strncasecmp(line + 1, "ontent-", 7) == 0) {
          if (strncasecmp(line+8, "type:", 5) == 0) {
            matched = 1;
          }
          else if (strncasecmp(line+8, "transfer-encoding:", 18) == 0) {
            matched = 1;
          }
          else if (strncasecmp(line+8, "md5:", 4) == 0) {
            matched = 1;
          }
          else if (strncasecmp(line+8, "length:", 7) == 0) {
            matched = 1;
          }
        }
        break;
      case 'f':
	if (strncasecmp(line+1, "rom ", 4) == 0) {
	  matched = 1;
	}
        else if (strncasecmp(line+1, "rom:", 4) == 0) {
          rfc822_parse_adrlist(&e->from, p, host);
          matched = 1;
        }
        break;
      case 'i':
        if (strncasecmp(line+1, "n-reply-to:", 11) == 0) {
          safe_free((void **)&e->in_reply_to);
          e->in_reply_to = safe_strdup(p);
          matched = 1;
        }
        break;
      case 'm':
        if (strncasecmp(line+1, "ime-version:", 12) == 0) {
          matched = 1;
        }
        break;
      case 'r':
        if (strncasecmp(line+1, "eferences:", 10) == 0) {
	  mutt_parse_references(&e->references, p);
          matched = 1;
        }
        break;
      case 's':
        if (strncasecmp (line+1, "ubject:", 7) == 0) {
          if (!e->subject) e->subject = safe_strdup(p);
          matched = 1;
        }
        else if (strncasecmp(line+1, "ender:", 6) == 0) {
          rfc822_parse_adrlist(&e->sender, p, host);
          matched = 1;
        }
	else if (strncasecmp(line+1, "tatus:", 6) == 0)
	  matched = 1;
        break;
      case 't':
        if (strncasecmp (line+1, "o:", 2) == 0) {
          rfc822_parse_adrlist(&e->to, p, host);
          matched = 1;
        }
        break;
      case 'x':
	if (strncasecmp(line+1, "-status:", 8) == 0)
	  matched = 1;
      }
      if (!matched && blen > 0) {
        /* Caller has requested that we keep track of "user-defined" headers */
        *b++ = safe_strdup(line);
        blen--;
        *b = 0;
      }
      if (buffer[0] == 0) break; /* end of headers */
      strcpy(line, buffer);
    }
    else
      strcpy(line, buffer);
  }
  return e;
}

#define ENV env->env

void ci_edit_headers (const char *body, METAENV *env)
{
  char combined[_POSIX_PATH_MAX]; /* tempfile used to edit headers + body */
  FILE *ifp, *ofp;
  char buffer[1024];
  int i;
  ENVELOPE *n;
  time_t mtime;
  struct stat st;

  mutt_mktemp(combined);

  if ((ofp = fopen(combined, "w")) == NULL) {
    dprint(1,(debugfile, "edit_headers(): %s: couldn't open for writing!\n",
              combined));
    mutt_error("Could not open temp file!");
    return;
  }

  mutt_write_rfc822_header(ofp, env, NULL, 1);
  fputc('\n', ofp); /* tie off the header. */

  /* now copy the body of the message. */
  ifp = fopen(body, "r");
  while ((i = fread (buffer, 1, sizeof (buffer), ifp)) > 0)
    fwrite (buffer, 1, i, ofp);

  fclose(ifp);
  fclose(ofp);

  stat(combined, &st);
  mtime = st.st_mtime;
  ci_edit_file(combined);
  stat(combined, &st);
  if (mtime == st.st_mtime) {
    dprint(1, (debugfile, "ci_edit_headers(): temp file was not modified.\n"));
    /* the file has not changed! */
    mutt_unlink(combined);
    return;
  }

  mutt_unlink(body);
  for (i=0; env->userhdrs[i]; i++) safe_free((void **)&env->userhdrs[i]);

  /* Read the temp file back in */
  ifp = fopen(combined, "r");
  ofp = fopen(body, "w");
  n = parse_header(ifp, 0, env->userhdrs, MAXUSERHDRS, "@");
  while ((i = fread(buffer, 1, sizeof (buffer), ifp)) > 0)
    fwrite(buffer, 1, i, ofp);
  fclose(ofp);
  fclose(ifp);
  mutt_unlink(combined);

  /* restore old info. */
  n->references = ENV->references;
  ENV->references = 0;
  n->in_reply_to = ENV->in_reply_to;
  ENV->in_reply_to = 0;
  mutt_free_envelope(&ENV);
  ENV = n;

  ENV->to = mutt_expand_aliases(ENV->to);
  ENV->cc = mutt_expand_aliases(ENV->cc);
  ENV->bcc = mutt_expand_aliases(ENV->bcc);

  if (ENV->from->next) {
    /*
     * User added a From: line, so remove the auto-generated one.
     */
    ADDRESS *t = ENV->from;

    ENV->from = ENV->from->next;
    t->next = 0;
    mutt_free_address(&t);
  }
}

#undef ENV
