#include "mutt.h"
#include "mutt_curses.h"
#include "keymap.h"

#include <string.h>

#define MACROFUNC(X) OP_MAIN_EXECUTE_MACRO

typedef struct macro_t {
  short menu;
  short key;
  unsigned char *sequence;
  struct macro_t *next;
} MACRO;

static MACRO *Macros = NULL;

void mutt_execute_macro (int menu, int key)
{
  MACRO *tmp = Macros;
  int len;
  unsigned char *cp;
  
  while (tmp) {
    if (menu ==  tmp->menu && key == tmp->key) {
      len = strlen(tmp->sequence);
      cp = tmp->sequence + len - 1;
      while (len > 0) {
	ungetch(*cp);
	cp--;
	len--;
      }
      return;
    }
    tmp = tmp->next;
  }
}

/* bind-macro main-menu <key> <sequence> */
int mutt_parse_macro (char *s, char *err, size_t errlen)
{
  MACRO *tmp;
  MACRO *ptr;
  int key;
  int menu;
  int count = 0;
  char *sequence;

  *err = 0;
  while ((s = strtokq(s, " \t\n")) != NULL) {
    switch(count) {
    case 0: /* menu name */
      if ((menu = mutt_check_menu(s)) == -1) {
	snprintf(err, errlen, "%s: no such menu", s);
	return(-1);
      }
      break;
    case 1: /* key */
      if ((key = mutt_check_key(s)) == -1) {
	snprintf(err, errlen, "%s: invalid key", s);
	return(-1);
      }
      break;
    case 2: /* key sequence */
      sequence = safe_strdup(s);
      break;
    default:
      break;
    }
    count++;
    s = NULL;
  }
  if (count < 3) {
    snprintf(err, errlen, "too few arguments to bind-macro");
    return(-1);
  }

  ptr = Macros;
  while (ptr) {
    if (ptr->menu == menu && ptr->key == key) {
      /* override the old macro */
      safe_free((void **)&ptr->sequence);
      ptr->sequence = sequence;
      return(0);
    }
    if (!ptr->next) break;
    ptr = ptr->next;
  }
  tmp = (MACRO *)safe_malloc(sizeof(MACRO));
  memset(tmp, 0, sizeof(MACRO));
  tmp->menu = menu;
  tmp->key = key;
  tmp->sequence = sequence;

  if (ptr)
    ptr->next = tmp;
  else
    Macros = tmp;
  keymap_bind(tmp->key, tmp->menu, MACROFUNC(tmp->menu));
  return(0);
}
